#include "defs.h"
#include "vdilcd.h"
#include <LiquidCrystal_I2C.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////
static const byte LT[8] = {  B00111,  B01111,  B11111,  B11111,  B11111,  B11111,  B11111,  B11111 };
static const byte UB[8] = {  B11111,  B11111,  B11111,  B00000,  B00000,  B00000,  B00000,  B00000 };
static const byte RT[8] = {  B11100,  B11110,  B11111,  B11111,  B11111,  B11111,  B11111,  B11111 };
static const byte LL[8] = {  B11111,  B11111,  B11111,  B11111,  B11111,  B11111,  B01111,  B00111 };
static const byte LB[8] = {  B00000,  B00000,  B00000,  B00000,  B00000,  B11111,  B11111,  B11111 };
static const byte LR[8] = {  B11111,  B11111,  B11111,  B11111,  B11111,  B11111,  B11110,  B11100 };
static const byte MB[8] = {  B11111,  B11111,  B11111,  B00000,  B00000,  B00000,  B11111,  B11111 };
static const byte block[8] = {  B11111,  B11111,  B11111,  B11111,  B11111,  B11111,  B11111,  B11111 };


//////////////////////////////////////////////////////////////////////////////////////////////////////
CVDILcd::CVDILcd(LiquidCrystal_I2C* lcd)
:m_lcd(lcd), m_currentVDIStyle(-1)
{
}

void CVDILcd::reset()
{
 	m_currentVDIStyle = -1;
}

void CVDILcd::updateVDI(int style, int phase, int strength, unsigned char signalValues[40], bool badReading)
{
    if(style == 0)
      paintStandardVDI(phase, strength, signalValues, badReading);
    else
      paintBigNumVDI(phase, strength, signalValues, badReading);    
}

void CVDILcd::paintBigNumVDI(int phase, int strength, unsigned char signalValues[40], bool badReading)
{  
  if(m_currentVDIStyle != 1)
  {
    createBigNumCustomChars();
    m_currentVDIStyle = 1;
  }
  
  m_lcd->clear();
  if(!badReading)
  {
    int x=0;
    if(phase<0)
    {
      bigMinus(x);  
      phase = -phase;
    }
    x+=4;  
    
    int digit1 = int(phase)/10;
     bigNumber(x, digit1);
    x+=4;  
    
    int digit0 = int(phase)%10;      
    bigNumber(x, digit0);

    //
    m_lcd->setCursor(13,0);
    int s = int(strength+0.5F);
    if(s<=99)
      m_lcd->print("0");
    if(s<=9)    
      m_lcd->print("0");
    m_lcd->print(s);             
  }
  else
  {
    int x=0;
    bigMinus(x);    
    x+=4;  
    bigMinus(x);
    x+=4;      
    bigMinus(x);

    m_lcd->setCursor(13,0);
    m_lcd->print("***");
  }
}

void CVDILcd::paintStandardVDI(int phase, int strength, unsigned char signalValues[40], bool badReading)
{  
    if(m_currentVDIStyle != 0)
      m_currentVDIStyle = 0;
    
    if(!badReading)
    {
      if(strength != 0)
      {
        m_lcd->setCursor(0,0);        
        m_lcd->print("ID:");
        m_lcd->print(phase>=0 ? "+" : "-");            
        if(phase>=-9 && phase<=9)
          m_lcd->print("0");      
        m_lcd->print(abs(int(phase)));    
        m_lcd->print("   ");

        m_lcd->setCursor(0,1);  
        m_lcd->print("SS:");      
        int s = int(strength+0.5F);
        if(s<=99)    
          m_lcd->print("0");
        if(s<=9)    
          m_lcd->print("0");
        m_lcd->print(s);             
        m_lcd->print("  ");      
        lcdGraph(8, 1, signalValues);        
      }
      else
      {
        m_lcd->setCursor(0,0);        
        m_lcd->print("ID:+00    ");
        m_lcd->setCursor(0,1);  
        m_lcd->print("SS:000  \x5f\x5f\x5f\x5f\x5f\x5f\x5f\x5f");
      }
    }
    else
    {
      m_lcd->setCursor(0,0);        
      m_lcd->print("ID:***    ");
      m_lcd->setCursor(0,1);        
      m_lcd->print("SS:***  \x5f\x5f\x5f\x5f\x5f\x5f\x5f\x5f");
    }

    updateBattGui();          
}

void CVDILcd::updateBattGui()
{
  float v = analogRead(BPin)/1024.0F * 5.0F * BATTERY_SCALER;      

  //paint batt voltage
  m_lcd->setCursor(10,0);    
  if(v<10.0F)    
    m_lcd->print("0");
  m_lcd->print(v);
  m_lcd->print("V");    
}

void CVDILcd::createBigNumCustomChars()
{
  // assignes each segment a write number
  m_lcd->createChar(0, (uint8_t*)LT);
  m_lcd->createChar(1, (uint8_t*)UB);
  m_lcd->createChar(2, (uint8_t*)RT);
  m_lcd->createChar(3, (uint8_t*)LL);
  m_lcd->createChar(4, (uint8_t*)LB);
  m_lcd->createChar(5, (uint8_t*)LR);
  m_lcd->createChar(6, (uint8_t*)MB);
  m_lcd->createChar(7, (uint8_t*)block);
}

void CVDILcd::bigMinus(int x)
{ // uses segments to build the -
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(4));
  m_lcd->write(byte(4));
  m_lcd->write(byte(4));
}

void CVDILcd::custom0(int x)
{ 
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(0));
  m_lcd->write(byte(1));
  m_lcd->write(byte(2));
  m_lcd->setCursor(x, 1);
  m_lcd->write(byte(3));
  m_lcd->write(byte(4));
  m_lcd->write(byte(5));
}

void CVDILcd::custom1(int x)
{
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(1));
  m_lcd->write(byte(2));
  m_lcd->setCursor(x,1);
  m_lcd->write(byte(4));
  m_lcd->write(byte(7));
  m_lcd->write(byte(4));
}

void CVDILcd::custom2(int x)
{
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(6));
  m_lcd->write(byte(6));
  m_lcd->write(byte(2));
  m_lcd->setCursor(x, 1);
  m_lcd->write(byte(3));
  m_lcd->write(byte(4));
  m_lcd->write(byte(4));
}

void CVDILcd::custom3(int x)
{
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(6));
  m_lcd->write(byte(6));
  m_lcd->write(byte(2));
  m_lcd->setCursor(x, 1);
  m_lcd->write(byte(4));
  m_lcd->write(byte(4));
  m_lcd->write(byte(5)); 
}

void CVDILcd::custom4(int x)
{
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(3));
  m_lcd->write(byte(4));
  m_lcd->write(byte(7));
  m_lcd->setCursor(x+2, 1);
  m_lcd->write(byte(7));
}

void CVDILcd::custom5(int x)
{
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(3));
  m_lcd->write(byte(6));
  m_lcd->write(byte(6));
  m_lcd->setCursor(x, 1);
  m_lcd->write(byte(4));
  m_lcd->write(byte(4));
  m_lcd->write(byte(5));
}

void CVDILcd::custom6(int x)
{
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(0));
  m_lcd->write(byte(6));
  m_lcd->write(byte(6));
  m_lcd->setCursor(x, 1);
  m_lcd->write(byte(3));
  m_lcd->write(byte(4));
  m_lcd->write(byte(5));
}

void CVDILcd::custom7(int x)
{
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(1));
  m_lcd->write(byte(1));
  m_lcd->write(byte(2));
  m_lcd->setCursor(x+2, 1);
  m_lcd->write(byte(7));
}

void CVDILcd::custom8(int x)
{
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(0));
  m_lcd->write(byte(6));
  m_lcd->write(byte(2));
  m_lcd->setCursor(x, 1);
  m_lcd->write(byte(3));
  m_lcd->write(byte(4));
  m_lcd->write(byte(5));
}

void CVDILcd::custom9(int x)
{
  m_lcd->setCursor(x,0);
  m_lcd->write(byte(0));
  m_lcd->write(byte(6));
  m_lcd->write(byte(2));
  m_lcd->setCursor(x+2, 1);
  m_lcd->write(byte(7));
}

void CVDILcd::bigNumber(int x, int n)
{
  switch(n)
  {
    case 0: custom0(x); break;
    case 1: custom1(x); break;
    case 2: custom2(x); break;
    case 3: custom3(x); break;
    case 4: custom4(x); break;
    case 5: custom5(x); break;
    case 6: custom6(x); break;
    case 7: custom7(x); break;
    case 8: custom8(x); break;
    case 9: custom9(x); break;   
  }
}

//data: array of 40 values 0 to 7 
void CVDILcd::lcdGraph(int row, int column, unsigned char data[40])
{
  unsigned char b[8][8] = {{0}};
  for(int n=0;n<40;n++)
  {
    int h = data[n];
    if(h>7) h=7;
    if(h<0) h=0;
    
    int c = n/5;
    int pc = 4-n%5;
    for(int r=7;r>=(7-h);r--)
    {    
      b[c][r] |= 1<<pc;
    }
  }
  
  for(int n=0;n<8;n++)
  {
    m_lcd->createChar(n, b[n]);        
  }
  
  m_lcd->setCursor(row, column);  
  for(int n=0;n<8;n++)
  {
    m_lcd->write(n);        
  }
}
