#include "defs.h"
#include "encoder.h"
#include "menu.h"
#include <MenuSystem.h>

extern MenuSystem ms;

volatile bool moving=false;
volatile unsigned int encValue = 0;
volatile bool enc1 = false;              
volatile bool enc2 = false;

///////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////
//Encoder
void intrEncChange1()
{
    if(moving)
        delay(1);
    if(digitalRead(PIN_ENC1) == enc1)
        return;
    enc1 = !enc1;
    if(enc1 && !enc2)
        encValue += 1;
    moving = false;
}

void intrEncChange2()
{
    if(moving)
        delay(1);
    if(digitalRead(PIN_ENC2) == enc2)
        return;
    enc2 = !enc2;
    if(enc2 && !enc1)
        encValue -= 1;
    moving = false;
}

void readEncoder(bool* prev, bool* next, bool* btnSelect, bool* btnBack) 
{
    static unsigned int encValueTracking = 1;    
    static unsigned long btnHeld = 0;
    
    moving = true;

    *prev = false;
    *next = false;
    *btnSelect = false;
    *btnBack = false;

    if(encValueTracking != encValue)
    {
        *prev = encValue<encValueTracking;
        *next = encValue>encValueTracking;        
        encValueTracking = encValue;
    }

    if((digitalRead(PIN_EBTN) == LOW) && !btnHeld)
    {
        btnHeld = millis();
    }

    long btnDownTime = 0;  
    if((digitalRead(PIN_EBTN) == HIGH) && btnHeld)
    {
        btnDownTime = millis() - btnHeld;
        *btnSelect = btnDownTime > 0 && btnDownTime < 500;         
        *btnBack = btnDownTime > 2000;   
        btnHeld = 0;
    }
}

bool checkEncoder()
{
    bool prev(false);
    bool next(false);
    bool btnSelect(false);
    bool btnBack(false);   
    readEncoder(&prev, &next, &btnSelect, &btnBack);  
          
    return btnSelect || btnBack || prev || next;
}

void encoderHandler() 
{
    bool prev(false);
    bool next(false);
    bool btnSelect(false);
    bool btnBack(false);   
    readEncoder(&prev, &next, &btnSelect, &btnBack);

    if(!prev && !next && !btnSelect && !btnBack)
        return;

    if(prev)
    { // Previus item
        ms.prev();
        displayVDIMenu();
    }else if (next)
    {
        ms.next();
        displayVDIMenu();
    }else if(btnSelect)
    {
        ms.select();      
        displayVDIMenu();
    }else if(btnBack)
    {
        ms.back();      
        displayVDIMenu();
    }
}

