#include <EEPROM.h>
#include "params.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////
//This function will write a 2 byte integer to the eeprom at the specified address and address + 1
void EEPROMWriteInt(int p_address, int p_value)
{
    unsigned char lowByte = ((p_value >> 0) & 0xFF);
    unsigned char highByte = ((p_value >> 8) & 0xFF);

    EEPROM.write(p_address, lowByte);
    EEPROM.write(p_address + 1, highByte);
}

//This function will read a 2 byte integer from the eeprom at the specified address and address + 1
unsigned int EEPROMReadInt(int p_address)
{
    unsigned char lowByte = EEPROM.read(p_address);
    unsigned char highByte = EEPROM.read(p_address + 1);
    return ((lowByte << 0) & 0xFF) + ((highByte << 8) & 0xFF00);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////
CParameters::CParameters()
:m_showVDI(true)
{
	setDefaults();
}

void CParameters::setDefaults()
{
      m_minSignal        = 10;
      m_XBias            = 400;
      m_RBias            = 400;
      m_maxSignal        = 1023;
      m_readingSamples   = 15;       
      m_LcdBacklight     = 1;
      m_vdiStyle         = 0;
      m_soundOn          = 0;
      m_sigPolarity      = 1; 

      m_showVDI          = true;
};

void CParameters::loadParams()
{
#define CURRENT_MAGIC  0xBAD3
    int magic = EEPROMReadInt(0);

    if(magic == CURRENT_MAGIC)
    {
        m_minSignal      = EEPROMReadInt(2);
        m_XBias          = EEPROMReadInt(4);
        m_RBias          = EEPROMReadInt(6);
        m_maxSignal      = EEPROMReadInt(8);
        m_readingSamples = EEPROMReadInt(10);
        m_LcdBacklight   = EEPROMReadInt(12);
        m_vdiStyle       = EEPROMReadInt(14);
        m_soundOn        = EEPROMReadInt(16);
        m_sigPolarity    = EEPROMReadInt(18);

    }
    else
    {
        setDefaults();

        EEPROMWriteInt(2, m_minSignal);
        EEPROMWriteInt(4, m_XBias);
        EEPROMWriteInt(6, m_RBias);
        EEPROMWriteInt(8, m_maxSignal);
        EEPROMWriteInt(10, m_readingSamples);                   
        EEPROMWriteInt(12, m_LcdBacklight);
        EEPROMWriteInt(14, m_vdiStyle);
        EEPROMWriteInt(16, m_soundOn);
        EEPROMWriteInt(18, m_sigPolarity);        

          
        EEPROMWriteInt(0, CURRENT_MAGIC);
    }
}

